"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailgunTransport = void 0;
const form_data_1 = __importDefault(require("form-data"));
const Requestly_1 = require("./services/Requestly");
const ADDRESS_KEYS = ['from', 'to', 'cc', 'bcc', 'replyTo'];
const CONTENT_KEYS = ['subject', 'text', 'html'];
const TRANSFORM_FIELDS = {
    replyTo: 'h:Reply-To',
};
const combineTarget = (target) => {
    return target.name ? `${target.name} <${target.address}>` : target.address;
};
class MailgunTransport {
    constructor(options) {
        this.name = 'MailgunTransport';
        this.version = 'N/A';
        this.requestConfig = {
            protocol: 'https:',
            hostname: options.hostname || 'api.mailgun.net',
            path: `/v3/${options.auth.domain}/messages`,
            auth: `api:${options.auth.apiKey}`,
        };
    }
    appendAddresses(form, data) {
        ADDRESS_KEYS.forEach((target) => {
            if (!data[target])
                return;
            let value;
            if (Array.isArray(data[target])) {
                value = data[target].map(combineTarget).join(',');
            }
            else {
                value = combineTarget(data[target]);
            }
            form.append(TRANSFORM_FIELDS[target] || target, value);
        });
    }
    appendContent(form, data) {
        CONTENT_KEYS.forEach((content) => {
            if (!data[content])
                return;
            form.append(content, data[content]);
        });
    }
    appendImages(form, data) {
        if (!Array.isArray(data.attachments))
            return;
        data.attachments.forEach((attachment) => {
            if (attachment.cid) {
                const buffer = Buffer.from(attachment.content, attachment.encoding);
                form.append('inline', buffer, {
                    filename: attachment.cid,
                    contentType: attachment.contentType,
                    knownLength: buffer.length,
                });
            }
        });
    }
    appendAttachments(form, data) {
        if (!Array.isArray(data.attachments))
            return;
        data.attachments.forEach((attachment) => {
            if (!attachment.cid) {
                const buffer = Buffer.from(attachment.content, attachment.encoding);
                form.append('attachment', buffer, {
                    filename: attachment.filename || attachment.cid,
                    contentType: attachment.contentType,
                    knownLength: buffer.length,
                });
            }
        });
    }
    submitForm(form) {
        return (0, Requestly_1.postForm)(this.requestConfig, form);
    }
    send(mail, done) {
        mail.normalize((error, data) => {
            if (error)
                return done(error);
            if (!data)
                return done(new Error('The email data is corrapted.'));
            const form = new form_data_1.default();
            this.appendAddresses(form, data);
            this.appendContent(form, data);
            this.appendImages(form, data);
            this.appendAttachments(form, data);
            this.submitForm(form)
                .then(() => {
                done(null, {
                    envelope: mail.message.getEnvelope(),
                    messageId: mail.message.messageId(),
                    message: form,
                });
            })
                .catch((e) => done(e));
        });
    }
}
exports.MailgunTransport = MailgunTransport;
exports.default = MailgunTransport;
